
# ======================================================================

# led.tcl

# Version   : 0.0.1
# Author    : Mark G. Saye
# Email     : markgsaye@yahoo.com
# Copyright : Copyright  2001
# Date      : November 13, 2001

# ======================================================================

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ======================================================================

  namespace eval led {}

# ======================================================================

proc led::after {w delay script} {

  debug 4 "proc led::after '$w' '$delay' '$script'"

# ----------------------------------------------------------------------

  variable led

# ----------------------------------------------------------------------

  set led($w,after,clicks) [clock clicks -milliseconds]
  set led($w,after,delay) $delay
  set led($w,after,id) [::after $delay $script]

# ----------------------------------------------------------------------

}

# ======================================================================

proc led::create {w args} {

  debug 2 "proc led::create '$w' '$args'"

# ----------------------------------------------------------------------

  label $w

# ----------------------------------------------------------------------

  set bg [$w cget -bg]
  set fg [$w cget -fg]

  foreach {r g b} [winfo rgb . $bg] { break }
  set data(-bg) [format #%01x%01x%01x \
    [expr $r / 256] [expr $g / 256] [expr $b / 256]]
  debug 3 "  bg='$bg' r,g,b='[winfo rgb . $bg]' rgb='$data(-bg)'"

  foreach {r g b} [winfo rgb . $fg] { break }
  set data(-fg) [format #%01x%01x%01x \
    [expr $r / 256] [expr $g / 256] [expr $b / 256]]
  debug 3 "  fg='$fg' r,g,b='[winfo rgb . $fg]' rgb='$data(-fg)'"

  set data(-width)  20
  set data(-height) 10
  set data(-relief) sunken

  array set data $args

# Ignore text option
  catch {unset data(-text)}

# ----------------------------------------------------------------------

  set image [image create photo \
    -width  $data(-width) \
    -height $data(-height) \
  ]

# ----------------------------------------------------------------------

  set data(-width)  0
  set data(-height) 0
  set data(-image) $image

# ----------------------------------------------------------------------

# $image put $bg -to 0 0 100 100
  $image put $data(-bg)

  eval $w configure [array get data]

  bind $w <Destroy> "led::destroy $w"

# ----------------------------------------------------------------------

  return $w

}

# ======================================================================

proc led::debug {level string} {

  variable debug

  if { $level > $debug } { return } { puts stderr "led: $string" }

}

# ======================================================================

proc led::destroy {w} {

  debug 2 "proc led::destroy '$w'"

# ----------------------------------------------------------------------

  variable led

# ----------------------------------------------------------------------

  if { [info exists led($w,after,id)] } {
    ::after cancel $led($w,after,id)
    foreach name [array names led "$w,*"] {
      unset led($name)
    }
  }

# ----------------------------------------------------------------------

}

# ======================================================================

proc led::flash {w on off} {

  debug 4 "proc led::flash '$w' '$on' '$off'"

# ----------------------------------------------------------------------

  variable led

# ----------------------------------------------------------------------

  led::toggle $w

# set led($w,after,id) [after $on "led::flash $w $off $on"]
  led::after $w $on "led::flash $w $off $on"

# ----------------------------------------------------------------------

}

# ======================================================================

proc led::init {} {

# ----------------------------------------------------------------------

  variable debug
  variable led

  if { ![info exists debug] } { set debug 1 }

  debug 2 "proc led::init"

# ----------------------------------------------------------------------

  set led(clock) [clock clicks -milliseconds]

# ----------------------------------------------------------------------

  namespace export led

# ----------------------------------------------------------------------

  package provide led 0.0.2

# ----------------------------------------------------------------------

}

# ======================================================================

proc led::led {args} { return [eval led::create $args] }

# ======================================================================

proc led::puts1 {args} {

# ----------------------------------------------------------------------

  variable led

# ----------------------------------------------------------------------

  set argv [lrange $args 0 end-1]
  set str  [lindex $args end]

# ::puts "[clock format [clock clicks -milliseconds] -format %H:%M:%S] $string"
  set dt [expr ([clock clicks -milliseconds] - $led(clock)) / 1000.0]
# ::puts "[clock format $dt -format %H:%M:%S] $string"
  eval ::puts $argv {"$dt $str"}

# ----------------------------------------------------------------------

}

# ======================================================================

proc led::reset {w} {

  debug 2 "proc led::reset '$w'"

# ----------------------------------------------------------------------

  variable led

# ----------------------------------------------------------------------

  if { [info exists led($w,after,id)] } {
    after cancel $led($w,after,id)
    unset led($w,after,id)
  }

# ----------------------------------------------------------------------

}

# ======================================================================

proc led::state {w newstate} {

  debug 2 "proc led::state '$w' '$newstate'"

# ----------------------------------------------------------------------

  variable led

# ----------------------------------------------------------------------

  set oldstate [$w cget -state]
  if { $oldstate == $newstate } {
#   puts "led:   already in state '$newstate'"
    return
  }

# ----------------------------------------------------------------------

  if { [string equal $oldstate disabled] } {
#   Reactivate flashing
    if { [info exists led($w,after,info)] } {
      set delay $led($w,after,delay)
      set script [lindex $led($w,after,info) 0]
      unset led($w,after,info)
      unset led($w,after,delay)
      led::after $w $delay $script
    }
#   Re-enable previous state
#   puts "led:   restoring to state '$led($w,state)'"
    $w configure -state $led($w,state)
    switch -- $led($w,state) {
      active {
        set bg $led($w,fg)
        set fg $led($w,bg)
      }
      normal {
        set bg $led($w,bg)
        set fg $led($w,fg)
      }
    }
#   puts "led:   restoring bg='$bg' fg='$fg'"

# ----------------------------------------------------------------------

  } elseif { [string equal $newstate disabled] } {
#   Save current state for when we re-enable
    set led($w,state) $oldstate
#   Find how much time has elapsed since invoking after
    if { [info exists led($w,after,id)] } {
      set id $led($w,after,id)
      set now [clock clicks -milliseconds]
      set dms [expr $now - $led($w,after,clicks)]
      set delay [expr $led($w,after,delay) - $dms]
#     puts "led:   dms='$dms' delay='$delay'"
      set led($w,after,delay) $delay
#     puts "led:   after info $w $id='[::after info $id]'"
      set led($w,after,info) [::after info $id]
      ::after cancel $id
      unset led($w,after,id)
    }
#   $w configure -state disabled
    switch -- $oldstate {
      active {
        set bg [$w cget -fg]
        set fg [$w cget -bg]
      }
      normal {
        set bg [$w cget -bg]
        set fg [$w cget -fg]
      }
    }
    set led($w,bg) $bg
    set led($w,fg) $fg
#   puts "led:   storing bg='$bg' fg='$fg'"

# ----------------------------------------------------------------------

  } else {
#   swap bg and fg colors
    set bg [$w cget -fg]
    set fg [$w cget -bg]
  }

# ----------------------------------------------------------------------

  set image [$w cget -image]
# $image put $bg -to [$image cget -width] [$image cget -height]
  $image put $bg

  $w configure \
    -bg $bg \
    -fg $fg \
    -activebackground $bg \
    -activeforeground $fg \
    -state $newstate

# ----------------------------------------------------------------------

}

# ======================================================================

proc led::toggle {w} {

  debug 2 "proc led::toggle '$w'"

# ----------------------------------------------------------------------

  variable led

# ----------------------------------------------------------------------

  set oldstate [$w cget -state]

  switch -- $oldstate {
    active {
      set newstate normal
    }
    disabled {
      return
    }
    normal {
      set newstate active
    }
  }

# puts "led:   state '$oldstate' -> '$newstate'"

  led::state $w $newstate

# ----------------------------------------------------------------------

}

# ======================================================================

  led::init

# ======================================================================

